--[[

	Speichern und Laden von RoM's Grafikeinstellungen
	
	Datei neu laden und alle alten Einstellungen löschen:
	/run dofile("Interface/Addons/RoM_Graphics/graphics.lua")
]]
local checkboxes = {
	Nutze_UI_Skalierung 					= SYS_DisplaySettings_UIScaleCheckButton,
	Nutze_Wasserreflekionen 				= SYS_DisplaySettings_WaterReflectionCheckButton,
	Nutze_Wasserbrechungen 					= SYS_DisplaySettings_WaterRefractionCheckButton,
	Nutze_Bloom 							= SYS_DisplaySettings_BloomCheckButton,
	Nutze_Punktreflektionen 				= SYS_DisplaySettings_SpecularHighlightCheckButton,
	Nutze_Auto_Anpassung 					= SYS_DisplaySettings_DebaseTextureCheckButton,
	Nutze_Verzerrungseffekte 				= SYS_DisplaySettings_DistortFXCheckButton,
	Nutze_Speicherwarnung_Deaktivieren 		= SYS_DisplaySettings_DisableMemoryWarningCheckButton,
	Nutze_Effekte_Ausblenden 				= SYS_DisplaySettings_HideOtherPlayerEffectCheckButton,
}

local sliders = {
	Wert_UI_Skalierung 						= SYS_DisplaySettings_UIScaleSlider,
	Wert_Gelaende_Shader 					= SYS_DisplaySettings_TerrainShaderDetailSlider,
	Wert_Schattendetails 					= SYS_DisplaySettings_ShadowDetailSlider,
	Wert_Charakterdetails 					= SYS_DisplaySettings_PaperdollDetailSlider,
	Wert_Sichtweite 						= SYS_DisplaySettings_ViewDistanceSlider,
	Wert_Weltdetails 						= SYS_DisplaySettings_WorldDetailSlider,
	Wert_Himmeldetails 						= SYS_DisplaySettings_SkyDetailSlider,
	Wert_Texturedetails 					= SYS_DisplaySettings_TextureDetailSlider,
	Wert_Gelaendedetails 					= SYS_DisplaySettings_TerrainTextureDetailSlider,
}

local ROM_Graphics = {
	SavedSettings = {},
	
	LoadSetting = function(name)
		if name and type(name) == "string" and name ~= "" then
			ExecuteMacroLine("/graphics load "..name)
		end
	end,
	
	ApplySettings = function(settings)
		if (not settings) or type(settings) ~= "table" then
			DEFAULT_CHAT_FRAME:AddMessage("|cffff0000Keine gültigen Einstellungen angegeben|r")
			return nil
		end
		for name, value in pairs(settings.checkboxes or {}) do
			local box = checkboxes[name]
			if box then
				if value == "an" or value == true then
					box:SetChecked(true)
				else
					box:SetChecked(false)
				end
			else
				DEFAULT_CHAT_FRAME:AddMessage("|cffff0000Es existiert keine CheckBox mit der Bezeichung '"..tostring(name).."'|r")
			end
		end
		for name, value in pairs(settings.sliders or {}) do
			local slider = sliders[name]
			if slider then
				local new_val = value
				local min = slider:GetMinValue()
				local max = slider:GetMaxValue()
				if value == "min" then
					new_val = min
				elseif value == "max" then
					new_val = max
				end
				
				new_val = math.max(min, math.min(max, new_val))
				
				if new_val then
					local func_name = (slider:GetName():gsub("SYS_DisplaySettings_", "", 1)).."_SetValue"
					if _G[func_name] then
						_G[func_name](new_val)
					end
					slider:SetValue(new_val)
				end
			else
				DEFAULT_CHAT_FRAME:AddMessage("|cffff0000Es existiert kein Regler mit der Bezeichung '"..tostring(name).."'|r")
			end
		end
		SSF_DisplaySettings_OnApply()
	end,
	
	SaveCurrentSetting = function(NewName)
		local rewrite_state = ROM_Graphics.SavedSettings[NewName] ~= nil
		
		ROM_Graphics.SavedSettings[NewName] = {
			checkboxes = {},
			sliders = {},
		}
		
		for index, box in pairs(checkboxes) do
			ROM_Graphics.SavedSettings[NewName].checkboxes[index] = (box:IsChecked() and "an") or "aus"
		end
		for index, slider in pairs(sliders) do
			if not slider.GetMinValue then
				return nil
			end
			local min = slider:GetMinValue()
			local max = slider:GetMaxValue()
			local cur = slider:GetValue()
			ROM_Graphics.SavedSettings[NewName].sliders[index] = (cur == min and "min") or (cur == max and "max") or string.format("%.2f", cur)
		end
		if rewrite_state then
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff00Profil '"..NewName.."' wird überschrieben|r")
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff00ff00Profil '"..NewName.."' wurde gespeichert|r")
		end
	end,
	
	OnEvent = function(event)
		if event == "VARIABLES_LOADED" then
			if ROM_Graphics_SaveVars then
				ROM_Graphics.SavedSettings = ROM_Graphics_SaveVars
			end
		elseif event == "SAVE_VARIABLES" then
			ROM_Graphics_SaveVars = ROM_Graphics.SavedSettings
			SaveVariables("ROM_Graphics_SaveVars")
		end
	end,
}

_G.ROM_Graphics = ROM_Graphics

SLASH_ROM_Graphics1 = "/graphics"
SlashCmdList["ROM_Graphics"] = function(editBox, msg)
	msg = msg or " list"

	local command = msg:match("%w+") or ""
	msg = msg:gsub(command, "", 1):sub(2)
	if command == "load" then
		if ROM_Graphics.SavedSettings[msg] then
			ROM_Graphics.ApplySettings(ROM_Graphics.SavedSettings[msg])
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffffKein Eintrag '"..tostring(msg).."' gespeichert.|r")
		end
	elseif command == "save" then
		if msg ~= "" then
			ROM_Graphics.SaveCurrentSetting(msg)
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffff'"..tostring(msg).."' ist kein gültiger Name.|r")
		end
	elseif command == "list" then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffffffFolgende Profile wurden hinterlegt:|r")
		for profil, _ in pairs(ROM_Graphics.SavedSettings) do
			DEFAULT_CHAT_FRAME:AddMessage("|c00000000-----|r|cffffffff"..profil.."|r")
		end
	elseif command == "clean" then
		ROM_Graphics.SavedSettings = {}
		DEFAULT_CHAT_FRAME:AddMessage("|cffffffffAlle gespeicherten Profile wurden gelöscht|r")
	elseif command == "remove" and msg and msg ~= "" then
		if ROM_Graphics.SavedSettings[msg] then
			ROM_Graphics.SavedSettings[msg] = nil
			DEFAULT_CHAT_FRAME:AddMessage("|cffffffffDas Profil '"..msg.."' wurden gelöscht|r")
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cffffffff/graphics list - Zeigt vorhandene Profile an|r")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffffff/graphics load Name - Lädt das Profil 'Name'|r")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffffff/graphics save Name - Speichert die aktuellen Einstellungen als 'Name'|r")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffffff/graphics remove Name - Löscht das Profil 'Name'|r")
		DEFAULT_CHAT_FRAME:AddMessage("|cffffffff/graphics clean - Löscht alle gespeicherten Profile|r")
	end
end